from pprint import pprint
from pathlib import Path
import oyaml as yaml
from cloudmesh.DEBUG import VERBOSE
from cloudmesh.common.util import readfile
from cloudmesh.common.dotdict import dotdict
from munch import munchify
from cloudmesh.common.FlatDict import flatten as dict_flatten
import os
from collections import Counter


class Document(dotdict):

    def __init__(self):
        self["title"] = None  # The title
        self["name"] = None  # A short name unique for the destination dir
        self["kind"] = "section"  # header section
        self["uri"] = None  # https://
        self["prefix"] = "http"  # https http local
        self["basename"] = None
        self["dirname"] = None
        self["destination"] = None  # where to copy it to
        self["counter"] = 0
        self["level"] = 0
        self["topic"] = None
        self["format"] = None
        self["indent"] = ""


class Documents(object):

    def __init__(self):
        self.entries = []
        self.depth = 0
        self.metadata = []

    def spec_replace(self,
                     book,
                     variables):

        variable_list = dict_flatten(variables, sep=".")

        spec = yaml.dump(book)

        for variable in variable_list:
            value = variable_list[variable]
            token = "{" + variable + "}"
            spec = spec.replace(token, value)

        output = yaml.load(spec, Loader=yaml.SafeLoader)
        return output

    def load(self, filename, base):
        filename = str(Path(filename).resolve())

        text_content = readfile(filename).strip()
        yaml_content = yaml.load(text_content, Loader=yaml.SafeLoader)

        self.metadata = yaml_content['metadata']
        #
        # REPLACE VARIABLES
        #
        book = yaml_content["BOOK"]
        variables = yaml_content
        del variables["BOOK"]

        self.flat = munchify(variables)
        b = self.spec_replace(book, variables)
        transformed = yaml.dump(b)
        # print (transformed)

        lines = transformed.split("\n")

        counter = 0
        for line in lines:
            if len(line) == 0 or line.strip().startswith("#"):
                continue
            doc = Document()
            indent, value = line.split("-", 1)
            indent = int(len(indent) / 2)
            self.depth = max(self.depth, indent)
            doc.level = indent
            doc.counter = counter
            if line.strip().endswith(":"):
                doc.kind = "header"
                doc.title = value.strip()[:-1]

            if doc.kind != "header":
                doc.uri = value.strip()
            self.entries.append(doc)
            counter += 1
        self.local_path_resolve()
        # self.make_local_files_unique()

    def __str__(self):
        data = str(self.__dict__)
        return (data)

    def local_path_resolve(self):
        path_list = [""] * (self.depth + 1)
        for entry in self.entries:
            if entry.kind == "section":

                if entry.uri.startswith("file://"):
                    entry.uri = entry.uri.replace("file://", "")
                    entry.uri = str(Path(entry.uri).resolve())
                entry.dirname = os.path.dirname(entry.uri)
                entry.basename = os.path.basename(entry.uri)
                entry.title, entry.format = entry.basename.split(".")
                entry.format = entry.format.lower()
                entry.path = '/'.join(path_list[:entry.level])

                entry.destination = self.metadata["dest"]
                entry.destination = str(Path(entry.destination).resolve())
                entry.destination = "{destination}/{path}/{basename}".format(
                    **entry)

            elif entry.kind == "header":
                path_list[entry.level] = entry.title
                entry.path = path_list[:entry.level + 1]

                entry.path = '/'.join(entry.path)

                entry.basename = "HEADER-{title}.md".format(**entry)
                entry.format = "md"

                entry.name = "HEADER-{title}".format(**entry)
                entry.prefix = None

                entry.destination = self.metadata["dest"]
                entry.destination = str(Path(entry.destination).resolve())
                # print (path_list)
                entry.destination = "{destination}/{path}/{basename}".format(
                    **entry)
                entry.dirname = os.path.dirname(entry.destination)

                # print(entry)

            entry.indent = "  " * entry.level

    def make_local_files_unique(self):
        ids = []
        locations = []
        for entry in self.entries:
            if entry.kind == "section":
                # print (entry)
                locations.append(entry.destination)
                ids.append(entry.counter)

        # print (ids)
        # pprint (locations)

        counts = Counter(locations)
        # pprint (counts)

        duplicates = {k: v for k, v in counts.items() if v > 1}

        # pprint (duplicates)
        for entry in self.entries:
            if entry.destination in duplicates:
                base, ending = entry.destination.rsplit(".", 1)
                counter = entry.counter
                entry.destination = str(Path(f"{base}-{counter}.{ending}"))



    def printer(self,
                section="{indent} - [ ] {title}",
                header="{indent} - [ ] {title}"
                ):
        content = []
        for entry in self.entries:
            if entry.kind == "header":
                kind = header
            elif entry.kind == "section":
                kind = section
            content.append(kind.format(**entry))
        return content



import os
import sys
from pathlib import Path
from pprint import pprint
from shutil import copyfile


import pkg_resources
import requests
from bookmanager.cover import Cover
from bookmanager.util import cat_bibfiles, find_unique_name
from bookmanager.util import create_metadata, create_css
from bookmanager.util import create_section
from bookmanager.util import filewrap
from bookmanager.util import download as page_download
from cloudmesh.DEBUG import VERBOSE
from cloudmesh.common.dotdict import dotdict
from cloudmesh.common.util import banner, path_expand, readfile
from tabulate import tabulate
from bookmanager.document import Documents

debug = False

# noinspection PyPep8
class Book:

    def __init__(self, arguments):

        self.docs = Documents()

        self.docs.load(arguments.YAML, "BOOK")


        self.metadata = self.docs.metadata
        self.arguments = arguments

    def printer(self,
                section="{counter:3} {level:3} {path:20} {indent} - [ ] {title}",
                header="{counter:3} {level:3} {path:20} {indent} - [ ] {title}"
                ):

        r = self.docs.printer(
                    section="{counter:3} {level:3} {path:20} {indent} - [ ] {title}",
                    header="{counter:3} {level:3} {path:20} {indent} - [ ] {title}"
                    )

        print('\n'.join(r))


    def list(self, output, details):
        banner("list")

        banner("MARDOWN")

        if output == "markdown":

            if details:
                self.printer(
                    section="{level:3} {path:20} {kind} {indent}- [ ] [{topic}]({url})",
                    header="{level:3} {path:20} {kind} {indent}- [ ]  {topic}",
                )
            else:
                self.printer(
                    section="{indent}- [ ] [{topic}]({url})",
                    header="{indent}- [ ]  {topic}",
                )



    def check(self):

        banner("check")

        for entry in self.docs.entries:
            if entry.kind == "section":
                if entry.uri.startswith("http"):
                    print(entry.uri, end="")
                    response = requests.get(entry.uri, headers={'Cache-Control': 'no-cache'})
                    if response.status_code < 400:
                        print(" -> ok")
                    else:
                        print("error", response.status_code)

    def download(self, force):

        banner("get")

        for entry in self.docs.entries:
            if entry["kind"] == "section":
                print(entry.level * "   ", entry.counter, entry.title, end=' ')
                print('...', end=' ')

                entry.destination = find_unique_name(entry, self.docs.entries)

                page_download(entry.uri, entry.destination, entry.level, force)
                print()
            elif entry.kind == 'header':
                print(entry.level * "   ", entry.counter, entry.title)

    def urls(self):

        banner("URL")

        if self.arguments.FORMAT in ["md", "markdown"]:
            kind = "text"
        else:
            kind = "list"

        urls = []
        for entry in self.docs.entries:
            if entry.kind == "section":

                urls.append(entry.uri)

        print('\n'.join(urls))


    def generate(self, output):

        banner(f"Creating {output}")

        files = []
        for entry in self.docs.entries:

            if entry.kind in ["section", "header"]:
                url = entry.url
                path = entry.path
                basename = entry.basename
                local = entry.destination
                files.append(local)

        banner("Finding Contents")

        print("Number of included Sections:", len(self.docs.entries))

        banner("Creating Command")

        files = " ".join(files)

        # metadata["stylesheet"] = path_expand(metadata["stylesheet"])
        title = self.metadata["title"]

        dirs = []
        for section in self.docs.entries:
            if section["kind"] == "section":
                # pprint(section)
                path = section["path"]
                dirs.append(path_expand(f"./dest/book/{path}"))
        dirs = set(dirs)
        # dirs = find_image_dirs(directory='./dest')

        create_metadata(self.metadata, "./dest/book/metadata.txt")
        create_css(self.metadata, "./dest/book/epub.css")

        directories = (":".join(dirs))
        metadata = path_expand("./dest/book/metadata.txt")
        filename = self.metadata["filename"]
        cat_bibfiles("./dest", "./dest/all.bib")

        bib = path_expand("./dest/all.bib")
        csl = path_expand("./dest/book/ieee-with-url.csl")
        bibfile = f"--filter pandoc-citeproc --metadata link-citations=true --bibliography={bib} --csl={csl}"
        all_bibs = readfile("./dest/all.bib")
        css_style = pkg_resources.resource_filename("bookmanager",
                                                    'template/epub/ieee-with-url.csl')

        copyfile(css_style, path_expand("./dest/book/ieee-with-url.csl"))

        if "@" not in all_bibs:
            bibfile = ""

        options = "--toc --toc-depth=6  --number-sections"
        resources = f"--resource-path={directories}"
        markdown = "--verbose --filter pandoc-crossref -f markdown+emoji --indented-code-classes=bash,python,yaml"

        if output in ["epub", "md", "markdown"]:

            epub = path_expand(f"./dest/{filename}")
            # noinspection PyPep8
            command = f'cd dest/book; pandoc {options} {markdown} {resources} {bibfile} -o {epub} {files} {metadata}'
            # pprint(command.split(" "))

        elif output == "pdf":
            # bug hard code for now
            pdf = path_expand(f"./dest/{filename}").replace(".epub", ".pdf")
            path= Path("../../bookmanager/bookmanager/template/latex/eisvogel").resolve()
            book= "-V titlepage=true"
            latex = f"--template {path} --pdf-engine=xelatex"
            command = f'pandoc {options} {markdown} {bibfile} {latex} {book} {resources} -o {pdf} {files} {metadata}'

        elif output == "html":
            metadata = "./dest/metadata.txt"
            options = "--toc --number-sections"
            command = f'pandoc {options} -o ./dest/book.html {files}'

        elif output == "docx":
            metadata = "./dest/metadata.txt"
            options = "--toc --number-sections"
            command = f'pandoc {options} -o ./dest/book.docx {files}'

        else:
            raise ValueError(f"this output format is not yet supported: {output}")

        VERBOSE(command)
        os.system(command)

    def level(self):

        def convert_level(entry):

            #destination = entry["destination"]
            #extension = os.path.basename(destination).rsplit(".", 1)[1]
            #if extension in ["yaml", "yml"]:
            #    VERBOSE(destination)
            #    filewrap(f"./dest/book/{destination}", "```", "```")

            pandoc_level = entry.level + 1
            tmp = str(Path(self.docs.metadata["dest"]) / "tmp.md")
            command = "pandoc --base-header-level={pandoc_level} -o {tmp} {destination} > log.txt".format(
                **entry, tmp=tmp, pandoc_level=pandoc_level)
            sys.stdout.flush()

            os.system(command)
            command = "cp {tmp} {destination} > log.txt".format(**entry, tmp=tmp)
            os.system(command)
            os.system(f"rm -f {tmp}")
            os.system("rm -f log.txt")


        banner("Creating Level")

        title = self.docs.metadata["title"]


        def PRINT(entry, end="\n"):
            print(entry.level * "   ", entry.counter, entry.title, entry.uri, end=end)


        # pprint(self.docs.entries)


        for entry in self.docs.entries:

            if entry.kind == "header":
                PRINT(entry)
                create_section(entry.destination, entry.title, entry.level + 1)

            elif entry.kind == "section":

                PRINT(entry, end=' ')
                sys.stdout.flush()

                # n = find_smalest_headers(readfile(entry["local"]))
                # print (n, end="")
                # out = reduce_headers(readfile(entry["local"]), n, entry.level)
                # writefile(entry["local"], "\n".join(out))

                print(entry.level, end="")
                convert_level(entry)

                print('...', end=' ')
                sys.stdout.flush()
                print()

    def cover(self):

        cover = Cover()

        metadata = dotdict(self.docs.metadata)

        pprint(metadata)
        image = metadata.image
        dest = metadata.dest

        image = path_expand(f"./{dest}/{image}")
        Path(os.path.dirname(image)).mkdir(parents=True, exist_ok=True)

        banner(f"Generating Cover Page: {image}")

        table = []
        for k, v in metadata.items():
            table.append([k, v])
        print(tabulate(table, tablefmt="grid",
                       headers=["Attrbute", "Value"]))

        cover.generate(
            image=image,
            background=metadata.background,
            title=metadata.title,
            subtitle=metadata.subtitle,
            author=metadata.author,
            email=metadata.email,
            webpage=metadata.url
        )



import os
from pathlib import Path
from colorama import Fore
from bookmanager.run import run


class Git(object):

    @staticmethod
    def url(repo):
        global repos
        if repo in repos['community'] or repo in repos['spring19']:
            return f"https://github.com/cloudmesh-community/{repo}"
        else:
            return f"https://github.com/cloudmesh/{repo}"

    @staticmethod
    def clone(repos):
        for repo in repos:
            print(f"clone -> {repo}", os.path.isdir(Path(f"./{repo}")))

            if not os.path.isdir(Path(f"./{repo}")):
                try:
                    location = Git.url(repo)
                    command = f"git clone {location}.git"
                    r = run(command)
                    print(f"         {r}")
                except Exception as e:
                    print(e)
            else:
                # noinspection PyPep8
                print(
                    Fore.RED + "         ERROR: not downlaoded as repo already exists.")

    @staticmethod
    def status(repos):
        for repo in repos:
            print("status ->", repo)
            os.chdir(repo)
            print(run("git status"))
            os.chdir("../")

    # git clone https://github.com/cloudmesh/get.git

    @staticmethod
    def pull(repos):
        for repo in repos:
            print("pull ->", repo)
            os.chdir(repo)
            print(run("git pull"))
            os.chdir("../")

    @staticmethod
    def install(repos, dev=False):
        for repo in repos:
            print("install ->", repo)
            if dev:
                os.chdir(repo)
                print(run("pip install -e ."))
                os.chdir("../")
            else:
                print(run("pip install {repo}".format(repo=repo)))



#!/usr/bin/env python

"""
Pandoc filter to create lists of all kinds
"""

from pandocfilters import toJSONFilters, walk, Str, Plain, Link, BulletList, Para, RawInline
from functools import reduce
from copy import deepcopy
import io
import sys
import codecs
import json
import re
import unicodedata
import subprocess

collections = {}
headers = [0, 0, 0, 0, 0, 0]
headers2 = [0, 0, 0, 0, 0, 0]

def stringify(x, format):
    """Walks the tree x and returns concatenated string content,
    leaving out all formatting.
    """
    result = []

    def go(key, val, format, meta):
        if key in ['Str', 'MetaString']:
            result.append(val)
        elif key == 'Code':
            result.append(val[1])
        elif key == 'Math':
            # Modified from the stringify function in the pandocfilter package
            if format == 'latex':
                result.append('$' + val[1] + '$')
            else:
                result.append(val[1])
        elif key == 'LineBreak':
            result.append(" ")
        elif key == 'Space':
            result.append(" ")
        elif key == 'Note':
            # Do not stringify value from Note node
            del val[:]

    walk(x, go, format, {})
    return ''.join(result)

def collect(key, value, format, meta):
    global headers

    # Is it a header? Keep the correct numbered headers in the headers array
    if key == 'Header':
        [level, [id, classes, attributes], content] = value
        if 'unnumbered' not in classes:
            headers[level - 1] = headers[level - 1] + 1
            for index in range(level, 6):
                headers[index] = 0

    # Is it a link with a right tag?
    elif key == 'Span':

        # Get the Span
        [[anchor, classes, other], text] = value

        # Is the anchor correct?
        result = re.match('^([a-zA-Z][\w.-]*):([\w.-]+)$', anchor)
        if result:
            global collections

            # Compute the name
            name = result.group(1)

            # Compute the identifier
            identifier = result.group(2)

            # Store the new item
            string = stringify(deepcopy(text), format)
            
            # Prepare the names
            names = []

            # Add the atomic name to the list
            names.append(name)

            # Prepare the latex output
            if format == 'latex':
               latex = '\\phantomsection\\addcontentsline{' + name + '}{figure}{' + string + '}'

            # Loop on all the headers
            for i in [0, 1, 2, 3, 4, 5]:
                if headers[i] > 0:
                    # Add an alternate name to the list
                    altName = name + ':' + '.'.join(map(str, headers[:i+1]))
                    names.append(altName)
                    if format == 'latex':
                       # Complete the latex output
                       latex = latex + '\\phantomsection\\addcontentsline{' + altName + '}{figure}{' + string + '}'
                       latex = latex + '\\phantomsection\\addcontentsline{' + altName + '_}{figure}{' + string + '}'
                else:
                    break

            for name in names:
                # Prepare the new collections if needed
                if name not in collections:
                    collections[name] = []
                collections[name].append({'identifier': identifier, 'text': string})

            # Special case for LaTeX output
            if format == 'latex':
                text.insert(0, RawInline('tex', latex))
                value[1] = text

def listof(key, value, format, meta):
    global headers2

    # Is it a header?
    if key == 'Header':
        [level, [id, classes, attributes], content] = value
        if 'unnumbered' not in classes:
            headers2[level - 1] = headers2[level - 1] + 1
            for index in range(level, 6):
                headers2[index] = 0

    # Is it a paragraph with only one string?
    if key == 'Para' and len(value) == 1 and value[0]['t'] == 'Str':

        # Is it {tag}?
        result = re.match('^{(?P<name>(?P<prefix>[a-zA-Z][\w.-]*)(?P<section>\:((?P<sharp>#(\.#)*)|(\d+(\.\d+)*)))?)}$', value[0]['c'])
        if result:

            prefix = result.group('prefix')

            # Get the collection name
            if result.group('sharp') == None:
                name = result.group('name')
            else:
                level = (len(result.group('sharp')) - 1) // 2 + 1
                name = prefix + ':' + '.'.join(map(str, headers2[:level]))

            # Is it an existing collection
            if name in collections:

                if format == 'latex':
                    # Special case for LaTeX output
                    if 'toccolor' in meta:
                        linkcolor = '\\hypersetup{linkcolor=' + stringify(meta['toccolor']['c'], format) + '}'
                    else:
                        linkcolor = '\\hypersetup{linkcolor=black}'
                    if result.group('sharp') == None:
                        suffix = ''
                    else:
                        suffix = '_'
                    return Para([RawInline('tex', linkcolor + '\\makeatletter\\@starttoc{' + name + suffix + '}\\makeatother')])

                else:
                    # Prepare the list
                    elements = []

                    # Loop on the collection
                    for value in collections[name]:

                        # Add an item to the list
                        if pandocVersion() < '1.16':
                            # pandoc 1.15
                            link = Link([Str(value['text'])], ['#' + prefix + ':' + value['identifier'], ''])
                        else:
                            # pandoc 1.16
                            link = Link(['', [], []], [Str(value['text'])], ['#' + prefix + ':' + value['identifier'], ''])

                        elements.append([Plain([link])])

                    # Return a bullet list
                    return BulletList(elements)

        # Special case where the paragraph start with '{{...'
        elif re.match('^{{[a-zA-Z][\w.-]*}$', value[0]['c']):
            value[0]['c'] = value[0]['c'][1:]

def pandocVersion():
    if not hasattr(pandocVersion, 'value'):
        p = subprocess.Popen(['pandoc', '-v'], stdout=subprocess.PIPE,stderr=subprocess.PIPE)
        out, err = p.communicate()
        pandocVersion.value = re.search(b'pandoc (?P<version>.*)', out).group('version').decode('utf-8')
    return pandocVersion.value


def main():
    toJSONFilters([collect, listof])

if __name__ == '__main__':
    main()




#!/usr/bin/env python

from pandocfilters import toJSONFilter, Emph, Para, Strong, Str

# pandoc -s -t native test.md
# pandoc test.md --filter ./header-list.py  -s  -t markdown

def behead(key, value, format, meta):
  if key == 'Header' and value[0] == 1:
    # return Para([Strong(value[2])])
    return Para([Strong(value[2])])
  elif key == 'Str':
    return Str(value.upper())


if __name__ == "__main__":
  toJSONFilter(behead)



from os.path import dirname
from pathlib import Path

import oyaml as yaml
# from bookmanager.yaml_parser_ok import json_flatten
from deprecated.yaml_parser import json_flatten
from cloudmesh.common.FlatDict import flatten as dict_flatten
# from cloudmesh.DEBUG import VERBOSE
from cloudmesh.common.util import path_expand
from munch import munchify


# noinspection PyPep8
class Config(object):
    __shared_state = {}

    def __init__(self, config='./toc.yaml'):
        """
        Initialize the Config class.

        :param config: A local file path to cloudmesh yaml config
            with a root element `cloudmesh`. Default: `.toc.yaml`
        
        Example:
        
        ---
        - BOOK:
          - "{b516}/preface.md"
          - INTRODUCTION:
            - "{manager}/other.md"
          - CLOUD:
            - "{manager}/other.md"
            - AWS:
              - "{manager}/other.md"
              - "i {manager}"
              - "r {manager}"
              - "p {manager}/test.py"
          - DEVOPS:
            - "{manager}/other.md"
            - ANSIBLE:
              - "{manager}/other.md"
              - "i {manager}"
              - "r {manager}"
              - "p {manager}/test.py"
        
        """

        self.__dict__ = self.__shared_state
        if "data" not in self.__dict__:
            # VERBOSE("Load config")

            self.config_path = Path(path_expand(config)).resolve()
            self.config_folder = dirname(self.config_path)

            with open(self.config_path, "r") as content:
                self.data = yaml.load(content, Loader=yaml.SafeLoader)

            self.variables = dict(self.data)
            keys = self.variables.keys()
            del self.variables['BOOK']

            self.book = list(self.data['BOOK'])
            self.flat = munchify(self.variables)
            self.book = self.spec_replace(self.book, self.variables)
            self.variables = dict_flatten(self.variables, sep=".")

    def spec_replace(self,
                     book,
                     variables):

        variable_list = dict_flatten(variables, sep=".")

        spec = yaml.dump(book)

        for variable in variable_list:
            value = variable_list[variable]
            token = "{" + variable + "}"
            spec = spec.replace(token, value)

        output = yaml.load(spec, Loader=yaml.SafeLoader)
        return output

    def flatten(self,
                book="BOOK",
                title="{book}",
                section="{parent}/{key}",
                header="{parent}/{key}",
                indent=""):
        result = json_flatten(self.book,
                              book=book,
                              title=title,
                              section=section,
                              header=header,
                              level=-1,
                              indent=indent)
        return result

    def dict(self):
        return self.data

    def output(self, result, kind="text"):
        content = []
        for entry in result:
            text = entry["output"].format(**entry)
            content.append(text)

        if kind in ["text"]:
            return '\n'.join(content)

        if kind in ['url']:
            out = []
            for entry in content:
                # print (entry)
                if entry != "":
                    out.append(entry)
            return out

        else:
            return content

    def __str__(self):
        return yaml.dump(self.data, default_flow_style=False, indent=2)

    def get(self, key, default=None):
        """
        A helper function for reading values from the config without
        a chain of `get()` calls.

        Usage:
            mongo_conn = conf.get('db.mongo.MONGO_CONNECTION_STRING')
            default_db = conf.get('default.db')
            az_credentials = conf.get('data.service.azure.credentials')

        :param default:
        :param key: A string representing the value's path in the config.
        """
        return self.data.get(key, default)

    def __getitem__(self, item):
        """
        gets an item form the dict. The key is . separated
        use it as follows get("a.b.c")
        :param item:
        :type item:
        :return:
        """
        if "." in item:
            keys = item.split(".")
        else:
            return self.data[item]
        element = self.data[keys[0]]
        for key in keys[1:]:
            element = element[key]
        return element



import os
import shutil
import subprocess
import sys
from pathlib import Path
from shutil import copyfile

import markdown
import oyaml as yaml
import pkg_resources
import requests
from cloudmesh.common.util import path_expand
from cloudmesh.common.util import writefile, readfile
from colorama import Fore
from markdown.extensions import Extension
from markdown.treeprocessors import Treeprocessor
from pprint import pprint
import copy
from cloudmesh.DEBUG import VERBOSE
from collections import Counter

def filewrap(path, prefix, postfix):
    content = readfile(path)
    if not content.startswith(prefix):
        content = prefix + "\n" + content
    if not content.endswith(postfix):
        content = content + "\n" + postfix

def find_unique_name(entry, entries):
    locations = []
    for element in entries:
        if element.kind == "section":
            # print (entry)
            locations.append(element.destination)

    counts = Counter(locations)

    duplicates = {k: v for k, v in counts.items() if v > 1}

    if entry.destination in duplicates:
        base, ending = entry.destination.rsplit(".", 1)
        counter = entry.counter
        entry.destination = str(Path(f"{base}-{counter}.{ending}"))
    return entry.destination

def cat_bibfiles(directory, output):
    d = path_expand(directory)
    bibs = list(Path(d).glob("**/*.bib"))

    pprint(bibs)

    r = ""
    for bib in bibs:
        bib = str(bib)
        content = readfile(bib)
        r = r + "\n\n% " + bib + "\n\n" + content
    writefile(output, r)

    return list(bibs)


def create_section(filename, header, n):
    writefile(filename, ("#" * n) + f" {header}\n\n")


def git_download(repo, path, destination):
    try:
        os.system(
            f"svn export https://github.com/{repo}/trunk/{path} {destination}")
    except:
        print("ERROR: file not found", repo, path)

def create_metadata(metadata, location):
    location = path_expand(location)
    Path(os.path.dirname(location)).mkdir(parents=True, exist_ok=True)
    if not os.path.isfile(location):
        metadata_file = pkg_resources.resource_filename(
            "bookmanager",
            'template/epub/metadata.txt')

        meta = copy.deepcopy(metadata)
        for field in ["author", "title"]:
            meta[field] = meta[field].replace("\n", " ")

        content = readfile(metadata_file)
        content = content.format(**meta)
        writefile(location, content)


def create_css(metadata, location):
    location = path_expand(location)
    Path(os.path.dirname(location)).mkdir(parents=True, exist_ok=True)
    if not os.path.isfile(location):
        css_file = pkg_resources.resource_filename("bookmanager",
                                                   'template/epub/epub.css')

        copyfile(css_file, location)


def find_image_dirs(directory='dest'):
    directory = path_expand(directory)
    directories = []

    p = Path(directory)
    images = []
    for t in ['png', "PNG", "JPG", "jpg", "JPEG", "jpeg"]:
        images = images + list(p.glob(f"**/*{t}"))
    dirs = []
    for image in images:
        dirs.append(os.path.dirname(image))
    dirs = set(dirs)
    return dirs


# First create the treeprocessor

class ImgExtractor(Treeprocessor):
    def run(self, doc):
        """Find all images and append to markdown.images. """
        self.markdown.images = []
        for image in doc.findall('.//img'):
            self.markdown.images.append(image.get('src'))


# Then tell markdown about it

class ImgExtExtension(Extension):
    def extendMarkdown(self, md, md_globals):
        img_ext = ImgExtractor(md)
        md.treeprocessors.add('imgext', img_ext, '>inline')


# Finally create an instance of the Markdown class with the new extension

def find_images(content):
    md = markdown.Markdown(extensions=[ImgExtExtension()])
    html = md.convert(content)
    return md.images


def find_smalest_headers(content):
    """
    find the smalest header level
    :param content:
    :type content:
    :return:
    :rtype:
    """
    ignore = False
    lines = content.split("\n")
    headers = []
    for line in lines:
        if line.startswith("`"):
            ignore = not ignore
            break
        if line.startswith("#") and not ignore:
            headers.append(line.split(" ")[0])
    headers = set(headers)
    level = sorted(headers, key=len)
    if len(level) == 0:
        level = 1
    else:
        level = len(level[0])
    return level


def reduce_headers(content, level, indent=1):
    """
    replaces the #* with level number of #

    :param content:
    :type content:
    :param level:
    :type level:
    :param indent:
    :type indent:
    :return:
    :rtype:
    """
    ignore = False
    lines = content.split("\n")
    headers = []
    out = []
    for line in lines:
        if line.startswith("`"):
            ignore = not ignore

        if line.startswith("#") and not ignore:
            line = line.replace("#" * level, "#")
            line = line.replace("# ", "#" * (indent - 1) + " ")

        out.append(line)
    return out


def get_file_from_git(url, directory, filename):
    d = Path(directory)
    d.mkdir(parents=True, exist_ok=True)
    r = requests.get(url, allow_redirects=True, headers={'Cache-Control': 'no-cache'})

    if r.status_code == 200:
        output = Path(directory) / filename
        with open(output, 'wb') as f:
            #if url.endswith(".bib"):
            #    VERBOSE(url)
            #    VERBOSE(output)
            #    VERBOSE(r.content)
            #    f.write(b"% " + url.encode('ascii') + "\n" +
            #            b"% " + output.encode('ascii') + "\n" +
            #            r.content)
            #    f.write("%" + url + "\n" + r.content)
            #else:
            f.write(r.content)
    else:
        if not url.endswith(".bib"):
            print()
            print("can not find")
            print()
            print(url)
            print("   ", directory, filename)
    return r


class Result(object):
    def __init__(self):
        self.content = None
        self.status_code = None


def get_file_from_local(url, directory, filename):
    d = Path(directory)
    d.mkdir(parents=True, exist_ok=True)

    output = Path(directory) / filename

    source = Path(url.replace("file://", "")).resolve()

    copyfile(source, output)

    r = Result()
    r.content = readfile(output).encode()
    r.status_code = "200"
    return r


def download(url, destination, level=0, force=False):
    destination = str(Path(destination).resolve())

    basename = os.path.basename(destination)
    directory = os.path.dirname(destination)

    filename = destination

    if os.path.exists(filename) and not force:
        print(Fore.RED + "Warning: file alredy exists" + Fore.RESET, end="")
        return

    content = ""
    if url.startswith("http"):
        r = get_file_from_git(url, directory, filename)
    elif url.startswith("file:"):
        r = get_file_from_local(url, directory, filename)
    else:
        print("prefix in url not supported", url)
        sys.exit(1)

    content = r.content

    #
    # DOWNLOAD IMAGES
    #
    # if type(content) == str:
    #    image_found = "![" in content
    #
    # else:
    image_found = b"![" in content

    if image_found:
        images = find_images(content)

        # from pprint import pprint ; pprint(images)
        print()
        print()
        print('   ' * (level + 2), "Downloading", len(images), " images")

        dirurl = os.path.dirname(url)
        for image in images:
            if not image.startswith("http"):
                print('   ' * (level + 2), "Download", image, end=" ")
                image_name = os.path.basename(image)
                image_url = f"{dirurl}/{image}"

                destination = f"{directory}"
                image_dir = os.path.dirname(f"{destination}/{image}")

                d = Path(image_dir)
                d.mkdir(parents=True, exist_ok=True)

                r = get_file_from_git(image_url, image_dir, image_name)
                print("...")
            else:
                print('   ' * (level + 2), "Skipping", image)

    #
    # DOWNLOAD BIB from guesssed bibfile
    #

    try:

        content = ""
        if url.startswith("http"):
            r = get_file_from_git(url, directory, filename)
        elif url.startswith("file:"):
            r = get_file_from_local(url, directory, filename)
        else:
            print("prefix in url not supported", url)
            sys.exit(1)

        if url.startswith("http"):
            bib_url_guess = str(url).split(".md")[0] + ".bib"
            bib_filename_guess = str(filename).split(".md")[0] + ".bib"

            check = get_file_from_git(bib_url_guess,
                                      directory,
                                      bib_filename_guess)

        elif url.startswith("file:"):

            bib_url_guess = str(url).split(".md")[0] + ".bib"
            bib_filename_guess = str(filename).split(".md")[0] + ".bib"

            check = get_file_from_local(url, directory, filename)
        else:
            print("prefix in url not supported", url)
            sys.exit(1)

        if check.status_code == 200:
            print('   ' * (level + 2), "Download",
                  os.path.basename(bib_filename_guess))
    except Exception as e:
        print(e)


def run(command):
    print(command)
    try:
        output = subprocess.check_output(command,
                                         shell=True,
                                         stderr=subprocess.STDOUT,
                                         )
    except subprocess.CalledProcessError as err:
        print(Fore.red + f"ERROR: {err}")
        sys.exit(1)

    return output.decode('utf-8')


def script(commands, environment):
    for command in commands:
        result = run(command.format(env=environment))
        print(result)


def yn_question(msg):
    while True:
        query = input(Fore.RED + msg)
        answer = query.lower().strip()
        if query == '' or answer not in ['yes', 'n']:
            print('Please answer with yes/n!')
        else:
            break
    return answer == 'yes'


def banner(txt, c=Fore.BLUE):
    """prints a banner of the form with a frame of # around the txt::

      ############################
      # txt
      ############################

    :param txt: a text message to be printed
    :type txt: string
    :param c: a char used for the frame
    :type c: char
    """
    print(c + "#" * 70)
    print(c + f"#{txt}")
    print(c + "#" * 70)


def remove(location):
    print("delete", location)
    try:
        shutil.rmtree(location)
    except Exception as e:
        print(e)


def readyaml(name):
    with open(name, 'r') as stream:
        try:
            d = yaml.safe_load(stream)
            return d
        except yaml.YAMLError as exc:
            print(exc)
    return []



"""bookmanager -- a helper to create books from mardown files in a yaml TOC.

Usage:
  bookmanager version
  bookmanager YAML cover
  bookmanager YAML get [--format=FORMAT] [--force]
  bookmanager YAML download
  bookmanager YAML level
  bookmanager YAML epub [--force]
  bookmanager YAML pdf [--force]
  bookmanager YAML html
  bookmanager YAML docx
  bookmanager YAML check [--format=FORMAT]
  bookmanager YAML urls [--format=FORMAT]
  bookmanager YAML list [--format=FORMAT] [--details]


Arguments:
  YAML   the yaml file

Options:
  -h --help
  -f, --format=FORMAT     [default: markdown]
  -d, --details           [default: False]

Description:

    In principal you only need one command at this time. All other commands are
    available for test purposes.

    You can create an epub with

      bookmanager YAML get [--format=FORMAT]

    The command searches for all images within the markdown document and fetches
    them so the document can be created locally with the images. We assume all
    images in the md document are for now not specified via http locations but
    via relative locations.

    To view the document use your favourite ePub Reader

    Other commands include:

      bookmanager YAML download [--format=FORMAT]

        downloads the urls into the ./dest directory for local processing

      bookmanager YAML check [--format=FORMAT]

        checks if the urls in the yaml file exist

      bookmanager YAML urls  [--format=FORMAT]

        lists all urls of the yaml file

      bookmanager YAML list [--format=FORMAT]

        lists the yaml file


    Not implemented are the following features:

    1) pdf:  bookmanager pdf book.yml

    YAML Table of Contents format:

      The table of contents for the book can be controlled with a simple yaml
      file that has some specific contextual enhancements. THis include the
      creation of a BOOK section that has the sections outlined in hierarchical
      form, and contains chapter and section headers without links that are
      automatically generated.

    Here is an example of a simple TOC yaml file:

    * https://github.com/cyberaide/bookmanager/blob/master/tests/python.yml

    Bugs and enhancement suggestions:

    * https://github.com/cyberaide/bookmanager/issues

"""
import os

from cloudmesh.common.dotdict import dotdict
from docopt import docopt
from bookmanager.book import Book
from bookmanager.__version__ import version
import sys

debug = False


def main():
    arguments = dotdict(docopt(__doc__))
    arguments["FORMAT"] = arguments["--format"] or "epub"
    force = arguments["--force"]
    # pprint(arguments)

    if arguments.version:
        print(version)
        sys.exit(0)

    book = Book(arguments)

    if arguments.force:
        os.system("rm -rf dest/book/epub.css")
        os.system("rm -rf dest/book/cover.png")
        os.system("rm -rf dest/bool/metadata.txt")

    if arguments.cover:

        book.cover()

    elif arguments.info:

        # pprint(config.book)
        # pprint(config.variables)

        raise NotImplementedError

    elif arguments.list and (arguments.FORMAT in ["md", "markdown"]):

        details = arguments["--details"]
        book.list("markdown", details)

    elif arguments.list and (arguments.FORMAT in ["list"]):

        book.list("list")

    elif arguments.check:

        book.check()

    elif arguments["get"]:

        output = arguments["--format"] or "epub"

        if output not in ["epub", "pdf", "md", "markdown"]:
            print(f"Error: format {output} not supported")
            sys.exit(1)

        book.cover()
        book.download(force)
        book.level()
        book.generate(output)

    elif arguments["download"]:

        book.download(force)

    elif arguments.urls:

        book.urls()

    elif arguments.epub:

        book.cover()
        book.generate("epub")

    elif arguments.pdf:

        book.cover()
        book.generate("pdf")

    elif arguments.html:

        book.generate("html")

    elif arguments.docx:

        book.generate("docx")

    elif arguments.level:

        book.level()


if __name__ == '__main__':
    main()



"""
Set all headers to level 1
"""

from panflute import *


def action(elem, doc):
    if isinstance(elem, Header):
        elem.level = 1

def main(doc=None):
    return run_filter(action, doc=doc)

if __name__ == '__main__':
    main()



#!/usr/bin/env python

"""
Pandoc filter to convert all level 2+ headers to paragraphs with
emphasized text.
"""

from pandocfilters import toJSONFilter, Emph, Para

def behead(key, value, format, meta):
  if key == 'Header' and value[0] == 1:
    return Para([Emph(value[2])])


if __name__ == "__main__":
  toJSONFilter(behead)



from PIL import Image, ImageDraw, ImageFont
from datetime import datetime
import pkg_resources


class Cover(object):

    def __init__(self):
        self.draw = None
        self.row = 1
        self.step = 50
        self.font = self.set_font("normal")
        self.color = 'rgb(255, 255, 255)'  # white color

    def set_font(self, scale):
        ttf = pkg_resources.resource_filename(
            "bookmanager",
            'template/epub/fonts/OpenSans-Bold.ttf')

        if scale == "large":
            self.font = ImageFont.truetype(ttf,
                                           size=int(self.step * .9))
        elif scale == "tiny":
            self.font = ImageFont.truetype(ttf,
                                           size=int(self.step * .3))
        elif scale == "small":
            self.font = ImageFont.truetype(
                ttf,
                size=int(self.step * .4))
        elif scale == "smaller":
            self.font = ImageFont.truetype(
                ttf,
                size=int(self.step * .3))

        elif scale == "credit":
            self.font = ImageFont.truetype(ttf,
                                           size=int(self.step * .25))

        else:
            self.font = ImageFont.truetype(ttf,
                                           size=int(self.step * .7))
        return self.font

    def reset(self, row=None):
        if row is None:
            self.row = 1
        else:
            self.row = row

    def skip(self, n):
        self.row = self.row + n

    def mesg(self, x, text, step=50, scale="normal"):
        lines = text.split("\n")
        for line in lines:
            font = self.set_font(scale)
            self.draw.text((self.step * x, self.step * self.row), line,
                           fill=self.color, font=self.font)
            self.row = self.row + 1

    def line(self, start, stop, width=10):

        self.draw.line((start * self.step, self.step * self.row,
                        stop * self.step, self.step * self.row),
                       fill=self.color,
                       width=width)

    def credit(self, n):
        self.mesg(n,
                  "Created by Cloudmesh & Cyberaide Bookmanager, "
                  "https://github.com/cyberaide/bookmanager ",
                  scale="credit")

    def generate(self,
                 image="cover.png",
                 background=None,
                 title="Title",
                 subtitle="Subtitle",
                 author='Author',
                 subauthor="Editor",
                 email="laszewski@gmail.com",
                 webpage="https://github.com/cyberaide/bookmanager",
                 date=None):

        if background is None:
            background = pkg_resources.resource_filename(
                "bookmanager",
                'template/epub/cover/cover-image.png')

        canvas = Image.open(background)
        self.draw = ImageDraw.Draw(canvas)

        if date is None:
            date = "{:%B %d, %Y - %I:%M %p}".format(datetime.now())

        self.reset()
        self.mesg(1, title, scale="large")
        self.mesg(1, subtitle, scale="normal")
        self.skip(2)
        self.line(1, 13)
        self.skip(1)
        self.mesg(2, author)
        self.mesg(2, subauthor, scale="tiny")
        self.skip(1)
        self.mesg(5, email, scale="small")
        self.skip(1)
        self.line(1, 13)
        self.skip(1)
        self.mesg(1, webpage, scale="tiny")

        self.reset(1)
        self.skip(16)

        self.mesg(2, date, scale="tiny")
        self.credit(1.5)

        canvas.save(image)



import subprocess
import sys
from cloudmesh.common.dotdict import dotdict
import os
from cloudmesh.DEBUG import VERBOSE


def run(cmd):
    out = dotdict({
        "stdout": None,
        "stderr": None,
        "code": None,

    })
    os.environ['PYTHONUNBUFFERED'] = "1"
    proc = subprocess.Popen(cmd,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.STDOUT,
                            )
    proc.wait()
    out.stdout, out.stderr = proc.communicate()
    out.code = proc.returncode

    if out.stderr is not None:
        out.stderr = out.stderr.decode("utf-8")

    if out.stdout is not None:
        out.stdout = out.stdout.decode("utf-8")

    return out



#!/usr/bin/env python
# ----------------------------------------------------------------------- #
# Copyright 2017, Gregor von Laszewski, Indiana University                #
#                                                                         #
# Licensed under the Apache License, Version 2.0 (the "License"); you may #
# not use this file except in compliance with the License. You may obtain #
# a copy of the License at                                                #
#                                                                         #
# http://www.apache.org/licenses/LICENSE-2.0                              #
#                                                                         #
# Unless required by applicable law or agreed to in writing, software     #
# distributed under the License is distributed on an "AS IS" BASIS,       #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.#
# See the License for the specific language governing permissions and     #
# limitations under the License.                                          #
# ------------------------------------------------------------------------#
"""
Cloudmesh CMD5 setup.
"""
import io

from setuptools import find_packages, setup


def readfile(filename):
    """
    Read a file
    :param filename: name of the file
    :return: returns the content of the file as string
    """
    with io.open(filename, encoding="utf-8") as stream:
        return stream.read()


requiers = """
oyaml
docopt
requests
colorama
tabulate
munch
pillow
markdown
cloudmesh-common
""".split("\n")

# dependency_links = ['http://github.com/nicolaiarocci/eve.git@develop']

version = readfile("VERSION").strip()

with open('README.md') as f:
    long_description = f.read()


NAME = "cyberaide-bookmanager"
DESCRIPTION = "Creating Books from content in different git and other cloud services."
AUTHOR = "Gregor von Laszewski"
AUTHOR_EMAIL = "laszewski@gmail.com"
URL = "https://github.com/cyberaide/bookmanager"


setup(
    name=NAME,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    version=version,
    license="Apache 2.0",
    url=URL,
    packages=find_packages(),
    keywords='pandoc book proceedings markdown epub',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Environment :: MacOS X",
        "Environment :: OpenStack",
        "Environment :: Other Environment",
        "Environment :: Plugins",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Operating System :: MacOS",
        "Operating System :: Microsoft :: Windows :: Windows 10",
        "Operating System :: Unix",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Topic :: Internet",
        "Topic :: Scientific/Engineering",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: User Interfaces",
        "Topic :: System",
        "Topic :: System :: Distributed Computing",
        "Topic :: System :: Shells",
        "Topic :: Utilities",
    ],
    include_package_data=True,
    install_requires=requiers,
    tests_require=[
        "flake8",
        "coverage",
        "bump2version",
        "wheel",
        "twine",
        "invoke",
        "pytest"
    ],
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'bookmanager = bookmanager.command:main',
        ],
    }
)
